/*                              .. $MMMMMMMMMMMMMN?..... .    
.                       ....IMMMMMMMMMMMMMMMMMMMMMMMMMMM>  
.                      ..MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM8  
.                  .  MMMMMMMMMMM~,..          . ~MMMMMMM.  
.            ....$MMMMMMMMMM,...                    . ...   
..MMMMMMMMMMMMMMMMMMMMMM$...                                
.,MMMMMMMMMMMMMMMMMMZ...              ... ....              
. MMMMMMMMMMM?.               . ..MMMMMMMMMMMMMM .          
.                           ..NMMMMMMMMMMMMMMMMMMM  .       
.                         IMMMMMM,..8MMMMMMMMMM:MMMM,.      
.     .    .. .     ...MMMMMI..   .MMMMMMMMMMM. MMMMM,.    
.MMMMMMMMMMMMMMMMMMMMMMMMMM ..     .MMMMMMMMMMM   ZMMMMM    
.MMMMMMMMMMMMMMMMMMMMMMMMMMM.      .,MMMMMMMMM.   . MMMM   
. :  ... ...       .....=MMMMMMMDI,  .IMMMMMM8NMMMMMMMMMMN  
.                          . .NMMMMMMMMMMMMMMM~         ..  
.                               ....MMMMMMMMM$              
.                                 MMMM= MMMMM:..            
.                              .IMMMM...MMMMMM .            
.      ...                    ?MMM8. . .~MMMM$..            
. ~MMMMMM8                 .MMMMM,.    . MMMM.              
. MMM.MMM                MMMMN.         MMMM.              
..MMM.MMM7          .. MMMMM             MMMM               
..MMMM  .        .. MMMMM .              NMMM.              
. .,MMM7 .   .  MMMMM....               IMMM.              
.    . MMMMMMMMMMM..                      MMM.              
.       ....  ..                         .MMM.              
.                                        .MMM               
.       Horus Audio                      .   .              
*/      
/*****************************************************************************/
// Filename     : STA310.c
// Author       : Olivier Swinkels                                           
// Description  : STA310 functions
/*****************************************************************************/


#pragma used+
//***************************************************************************//
// STA310  initialisation routine 
//***************************************************************************//
s8 sta310_init(u8 i2c_addr)
{
    //release hard reset
    RELEASE_DIG_AUDIO_RESET;
    // TODO  insert delay
    // Soft reset
    sta310_softReset(i2c_addr);  
   
    // Audio PLL configuration
    sta310_audioPllConf(i2c_addr,0); 
    i2c_write_reg(i2c_addr,0xB5,0x01);      //audio PLL enable
    
    // enable all interrupts
    i2c_write_reg(i2c_addr,0x07,0xFF);   
    i2c_write_reg(i2c_addr,0x08,0xFF);   
    
    // Synchronisation configuration
    sta310_syncConf(i2c_addr);
    
    // select SPDIF pins as external audio input 
    i2c_write_reg(i2c_addr,0x0C,0x0B);   
    i2c_write_reg(i2c_addr,0x0D,0x08);  
 
    //autodetect enable
    i2c_write_reg(i2c_addr,0xE0,0x01);   
    
    //PCM configuration for PCM1792
    i2c_write_reg(i2c_addr,0x55,0x23); // I2S 24 bit
    i2c_write_reg(i2c_addr,0x54,0x01); // 32 bit mode 256 Fs
    
    i2c_write_reg(i2c_addr,0x05,0x01); // 44.1 kHz
    i2c_write_reg(i2c_addr,0xA8,0x01); // DVD standart spec
    i2c_write_reg(i2c_addr,0xA9,0x00); // stereo PCM

    //DEBUG select sine generator
    //i2c_write_reg(i2c_addr,0x05,0x01);     
    //i2c_write_reg(i2c_addr,0x68,0x0A);    //should approx 2kHz tone @ Fs =46
    //sta310_selectDecoder(i2c_addr, DEC_SPDIF_DD);   //beep tone
    sta310_selectDecoder(i2c_addr, DEC_ES_PCM);   //PCM
    //sta310_selectDecoder(i2c_addr, DEC_PINK_NOISE);   //PCM
    
    // no post processing
    i2c_read_reg(i2c_addr,0x65); //no pro logic  
    i2c_read_reg(i2c_addr,0x57); //channel delay  
    i2c_read_reg(i2c_addr,0x58); //channel delay  
    i2c_read_reg(i2c_addr,0x59); //channel delay  
    i2c_read_reg(i2c_addr,0x5A); //channel delay  
    i2c_read_reg(i2c_addr,0x5B); //channel delay  
    i2c_read_reg(i2c_addr,0x5C); //channel delay  
    i2c_read_reg(i2c_addr,0xAF); //channel delay  
    i2c_read_reg(i2c_addr,0xB0); //channel delay  
    i2c_write_reg(i2c_addr,0xB1,0x00); //PDEC2  
    i2c_write_reg(i2c_addr,0x62,0x00); //no prologic post processing  
    
    // reset effect parameters
    i2c_read_reg(i2c_addr,0x64); //      
    i2c_write_reg(i2c_addr,0x64,0x04); //no prologic auto balance  
    i2c_write_reg(i2c_addr,0x65,0x00); //no prologic downmix  
    i2c_write_reg(i2c_addr,0xAB,0x00); //?? (should be set according to  getting started guide.. )
    i2c_write_reg(i2c_addr,0xB2,0x00); //??
    i2c_write_reg(i2c_addr,0xB3,0x00); //??
    i2c_write_reg(i2c_addr,0x66,0x00); // no bass boost
    
    // NO volume configuration
    
    // NO channel cross
    i2c_write_reg(i2c_addr,0x56,0x00); //
    i2c_write_reg(i2c_addr,0x7F,0x01); //
  
    // Run and start playing music
    i2c_write_reg(i2c_addr,0x13,0x01); // play 
    i2c_write_reg(i2c_addr,0x14,0x00); // no mute
    i2c_write_reg(i2c_addr,0x72,0x01); // run   
    
    return 0;   // all ok    
};



//***************************************************************************//
// Function:    sta310_getFs
// Description: Gets the sample frequency of the current stream through the 
//              selected STA310
//***************************************************************************//
u8 sta310_getFs(u8 i2c_addr){

    switch(i2c_read_reg(i2c_addr,0x05) ){
        case 0:  return 46;  break;
        case 1:  return 44;  break;
        case 2:  return 32;  break;
        case 4:  return 96;  break;
        case 5:  return 88;  break;
        case 6:  return 64;  break;
        case 8:  return 24;  break;
        case 9:  return 22;  break;
        case 10: return 16;  break;
        case 12: return 12;  break;
        case 13: return 11;  break;
        case 14: return 8;   break;
        case 16: return 192; break;
        case 17: return 176; break;
        case 18: return 128; break;
        default: return 0;
    }    
}

//***************************************************************************//
// Function:    sta310_softReset
// Description: Executes a soft reset on the selected STA310
//***************************************************************************//
s8 sta310_softReset(u8 i2c_addr){
    int timeout=0;
    // Soft reset
    i2c_write_reg(i2c_addr,0x10,0x01);     
    i2c_write_reg(i2c_addr,0x2B,0x08);
    i2c_write_reg(i2c_addr,0x3A,0x00);
    while(i2c_read_reg(i2c_addr,0xFF) != 0x1){
        if(timeout++ == 10000){
            error_msg("STA310 init failed!");
            return -1;      // error  
        }
    }
    return 0;   // all ok    
}
//***************************************************************************//
// Function:    sta310_audioPllConf
// Description: Configure audio PLL
//***************************************************************************//
s8 sta310_audioPllConf(u8 i2c_addr,u8 source){
    
    switch(source){
        case 0:     // use 27Mhz system clock to create 256xFs PCMLCK
            i2c_write_reg(i2c_addr,0xB6,0x34); 
            i2c_write_reg(i2c_addr,0xB7,0xEC);
            i2c_write_reg(i2c_addr,0xB8,0x02);
            i2c_write_reg(i2c_addr,0xB9,0x09);
            i2c_write_reg(i2c_addr,0xBA,0x01);
            i2c_write_reg(i2c_addr,0xBB,0x03);
            i2c_write_reg(i2c_addr,0xBC,0x09);
            i2c_write_reg(i2c_addr,0xBD,0x02);
            i2c_write_reg(i2c_addr,0xBE,0x09);
            i2c_write_reg(i2c_addr,0xBF,0x01);
        break;
        default:
            // not used at the moment
        break;
    }
   
    return 0;       
}

//***************************************************************************//
// STA310 Select decoder:
// 00:DEC_PES_DD           :0 0 MPEG2 PES carrying Dolby Digital (ATSC)
// 01:DEC_PES_MPEG1        :0 1 MPEG2 PES carrying MPEG1 frames
// 02:DEC_PES_MPEG2        :0 2 MPEG2 PES carrying MPEG2 frames
// 03:DEC_PES_DD_DVD       :1 0 MPEG2 PES carrying Dolby Digital frames for DVD Video
// 04:DEC_PES_MPEG1_DVD    :1 2 MPEG2 PES carrying MPEG2 frames for DVD Video
// 05:DEC_PES_PCM_DVD      :1 3 MPEG2 PES carrying Linear PCM frames for DVD Video
// 06:DEC_PACKET_MPEG1     :1 1 MPEG1 packet carrying MPEG1 frames
// 07:DEC_ES_DD            :3 0 Dolby Digital frame elementary streams
// 08:DEC_ES_MPEG1         :3 1 MPEG1 frame elementary streams
// 09:DEC_ES_MPEG2         :3 2 MPEG2 frame elementary streams
// 10:DEC_ES_PCM           :3 3 Stereo PCM (16bits samples)
// 11:DEC_PINK_NOISE       :3 4 Pink Noise Generator
// 12:DEC_ES_CDDA          :3 5 CDDA frames
// 13:DEC_BEEP_TONE        :3 7 Beep Tone Generator
// 14:DEC_ES_MP3           :3 9 MP3 frame elementary streams
// 15:DEC_SPDIF_DD         :5 0 IEC61937 Input with Dolby Digital frames
// 16:DEC_SPDIF_MPEG1      :5 1 IEC61937 Input with MPEG1 frames
// 17:DEC_SPDIF_MPEG2      :5 2 IEC61937 Input with MPEG2 frames
// 18:DEC_SPDIF_DTS        :5 6 IEC61937 Input with DTS frames
// 19:DEC_PES_DVD_AUDIO    :6 3 MPEG2 PES carrying Linear PCM for DVD Audio
// 20:DEC_PES_MPL_DVD_AUDIO:6 8 MPEG2 PES carrying MPL for DVD Audio
//***************************************************************************//
s8 sta310_selectDecoder(u8 i2c_addr, u8 decoder){
    switch (decoder){
        case DEC_PES_DD: //0 0 MPEG2 PES carrying Dolby Digital (ATSC)
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,0);    
            i2c_write_reg(i2c_addr,DECODESEL,0); 
        break;            
        case DEC_PES_MPEG1: //0 1 MPEG2 PES carrying MPEG1 frames
            EXTERNAL_AUDIO_CLOCK
            i2c_write_reg(i2c_addr,STREAMSEL,0);    
            i2c_write_reg(i2c_addr,DECODESEL,1);    
        break;
        case DEC_PES_MPEG2: //0 2 MPEG2 PES carrying MPEG2 frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,0);    
            i2c_write_reg(i2c_addr,DECODESEL,2);    
        break;
        case DEC_PES_DD_DVD: //1 0 MPEG2 PES carrying Dolby Digital frames for DVD Video
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,1);    
            i2c_write_reg(i2c_addr,DECODESEL,0);    
        break;
        case DEC_PES_MPEG1_DVD: //1 2 MPEG2 PES carrying MPEG2 frames for DVD Video
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,1);    
            i2c_write_reg(i2c_addr,DECODESEL,2);    
        break;
        case DEC_PES_PCM_DVD: //1 3 MPEG2 PES carrying Linear PCM frames for DVD Video
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,1);    
            i2c_write_reg(i2c_addr,DECODESEL,3);    
        break;
        case DEC_PACKET_MPEG1: //1 1 MPEG1 packet carrying MPEG1 frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,2);    
            i2c_write_reg(i2c_addr,DECODESEL,1);    
        break;
        case DEC_ES_DD: //3 0 Dolby Digital frame elementary streams
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,0);    
        break;
        case DEC_ES_MPEG1: //3 1 MPEG1 frame elementary streams
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,1);    
        break;
        case DEC_ES_MPEG2: //3 2 MPEG2 frame elementary streams
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,2);    
        break;
        case DEC_ES_PCM: //3 3 Stereo PCM (16bits samples)
            EXTERNAL_AUDIO_CLOCK;
            //INTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,3);    
        break;
        case DEC_PINK_NOISE: //3 4 Pink Noise Generator
            INTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,4);    
        break;
        case DEC_ES_CDDA: //3 5 CDDA frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,5);    
        break;
        case DEC_BEEP_TONE: //3 7 Beep Tone Generator
            INTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,7);    
        break;
        case DEC_ES_MP3: //3 9 MP3 frame elementary streams
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,3);    
            i2c_write_reg(i2c_addr,DECODESEL,9);    
        break;
        case DEC_SPDIF_DD: //5 0 IEC61937 Input with Dolby Digital frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,5);    
            i2c_write_reg(i2c_addr,DECODESEL,0);    
        break;
        case DEC_SPDIF_MPEG1: //5 1 IEC61937 Input with MPEG1 frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,5);    
            i2c_write_reg(i2c_addr,DECODESEL,1);    
        break;
        case DEC_SPDIF_MPEG2: //5 2 IEC61937 Input with MPEG2 frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,5);    
            i2c_write_reg(i2c_addr,DECODESEL,2);    
        break;
        case DEC_SPDIF_DTS: //5 6 IEC61937 Input with DTS frames
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,5);    
            i2c_write_reg(i2c_addr,DECODESEL,6);    
        break;
        case DEC_PES_DVD_AUDIO: //6 3 MPEG2 PES carrying Linear PCM for DVD Audio
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,6);    
            i2c_write_reg(i2c_addr,DECODESEL,3);    
        break;
        case DEC_PES_MPL_DVD_AUDIO: //6 8 MPEG2 PES carrying MPL for DVD Audio    
            EXTERNAL_AUDIO_CLOCK;
            i2c_write_reg(i2c_addr,STREAMSEL,6);    
            i2c_write_reg(i2c_addr,DECODESEL,8); 
        break;
        default:
            error_msg("illegal decoder selected!");
        break;
    }
    return 0;       
}

//***************************************************************************//
// Synchonistation registers 
// Loads default values.
//***************************************************************************//
s8 sta310_syncConf(u8 i2c_addr){
    i2c_write_reg(i2c_addr,0x4F,0x00);     
    i2c_write_reg(i2c_addr,0x53,0x00);     
    i2c_write_reg(i2c_addr,0x50,0x00);     
    i2c_write_reg(i2c_addr,0x51,0x00);     
    i2c_write_reg(i2c_addr,0x52,0x00);     
    return 0;
}

#pragma used-
